/* Copyright (C) Yutaka Blizman, 2008.
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the copyright text
 * below is included in the resulting source code, for example:
 * "Portions Copyright (C) Yutaka Blizman, 2008"
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef struct
{	char key;
	int size;
	char* value[13];
}map;

typedef struct
{	int size;
	map row[26];
}table;

char *dict[] =
{ "aeroplane","afternoon","agent","alcohol","animal","bagpiper","bagpipes","barnacle","beach","bed","beer","before","bicycle","bill","bird","blond","boarding","border","bread","bridge","broth","bus","buy","camp","castle","cathedral","caw","celtic","century","clam","clean","closed","clothing","cod","cold","corn","corridor","counter","crayfish","crossroads","cup","customs","chair","change","chapel","cheese","chemist","chicken","danger","dark","day","difficult","disco","dish","ditch","doctor","dog","dolmen","door","down","easy","egg","eight","employee","far","find","fire","first","fish","five","flight","fog","fork","four","friend","friend","fruit","fur","galician","galician","garbage","garlic","get","glass","go","granary","green","hake","ham","hand","handkerchief","have","have","have","here","horse","hot","hotel","how","hundred","ice","inside","island","jewel","key","knife","land","late","laugh","left","left","less","letter","light","lost","love","luggage",0
};

// compares two char arrays to see if they are the same
int compare(char* c1, char* c2)
{	unsigned int i;
	for (i=0; i<strlen(c2); ++i)
	{	if (c1[i] == c2[i]);
		else return 0;
	}
	return 1;
}

// if the parity of every word for a char choice matches, it is guaranteed a win
int guaranteedWin(char* array[], int size, int parity)
{	int i;
	for (i=0; i<size; ++i)
	{	if (strlen(array[i]) % 2 == parity);
		else return 0;
	}
	return 1;
}

// if the parity of every word for a char choice does NOT match, it is guaranteed a loss
int guaranteedLose(char* array[], int size, int parity)
{	int i;
	for (i=0; i<size; ++i)
	{	if (strlen(array[i]) % 2 != parity);
		else return 0;
	}
	return 1;
}

// checks an array to see if any possible wins exist
int keepAlive(char* array[], int size, int parity)
{	int i;
	for (i=0; i<size; ++i)
	{	if (strlen(array[i]) % 2 == parity)
			return 1;
	}
	return 0;
}

// takes a substring and a dictionary and prints out:
// - the list of guaranteed win characters and their associated words
// - the list of guaranteed lose characters and their words
// - the list of keep-alive letters which may result in a win, but excluding guaranteed lose words if that letter is chosen.
void ghost(char* substring, char* dictionary[])
{	int i, j;
	int done = 0, dictIndex = 0;
	int substringLen = strlen(substring);
	int substringParity = substringLen % 2;
	table candidates, wins, loses, keepAlives;
	candidates.size = 0, wins.size = 0, loses.size = 0, keepAlives.size = 0;

	// fill table of candidates
	while (!done)
	{	if (dictionary[dictIndex] == NULL)
			done = 1;
		else
		{	if (compare(dictionary[dictIndex], substring))
			{	if (dictionary[dictIndex][substringLen] == candidates.row[candidates.size-1].key) // add a chain
				{	candidates.row[candidates.size-1].value[candidates.row[candidates.size-1].size] = dictionary[dictIndex];
					++(candidates.row[candidates.size-1].size);
				}
				else // add a row
				{	candidates.row[candidates.size].size = 0;
					candidates.row[candidates.size].key = dictionary[dictIndex][substringLen];
					candidates.row[candidates.size].value[0] = dictionary[dictIndex];
					++(candidates.row[candidates.size].size);
					++(candidates.size);
				}
			}
			++dictIndex;
		}
	}

	/////// DEBUG ///////
	printf("\n");
	for (i=0; i<candidates.size; ++i)
	{	printf("%c => \"%s\"", candidates.row[i].key, candidates.row[i].value[0]);
		for (j=1; j<candidates.row[i].size; ++j)
			printf(", \"%s\"", candidates.row[i].value[j]);
		printf("\n");
	}
	/////// END DEBUG ///////

	// fill table of guaranteed wins
	for (i=0; i<candidates.size; ++i)
	{	if (guaranteedWin(candidates.row[i].value, candidates.row[i].size, substringParity))
		{	wins.row[wins.size] = candidates.row[i];
			++(wins.size);
			candidates.row[i].key = NULL;
		}
	}

	// fill table of guaranteed loses
	for (i=0; i<candidates.size; ++i)
	{	if (guaranteedLose(candidates.row[i].value, candidates.row[i].size, substringParity))
		{	loses.row[loses.size] = candidates.row[i];
			++(loses.size);
			candidates.row[i].key = NULL;
		}
	}

	// fill table of keep-alives
	for (i=0; i<candidates.size; ++i)
	{	if (candidates.row[i].key == NULL); // ignore this row
		else
		{	if (keepAlive(candidates.row[i].value, candidates.row[i].size, substringParity))
			{	for (j=0; j<candidates.row[i].size; ++j)
				{	if (strlen(candidates.row[i].value[j]) % 2 == substringParity)
					{	keepAlives.row[keepAlives.size].key = candidates.row[i].key;
						keepAlives.row[keepAlives.size].value[0] = candidates.row[i].value[j];
						keepAlives.row[keepAlives.size].size = 1;
						++(keepAlives.size);
					}
				}
			}
		}
	}

	// print all guaranteed wins
	printf("\nguaranteed win  : ");
	if (wins.size == 0)
		printf("-none-");
	else
	{	for (i=0; i<wins.size; ++i)
		{	printf("%c => \"%s\"", wins.row[i].key, wins.row[i].value[0]);
			for (j=1; j<wins.row[i].size; ++j)
				printf(", \"%s\"", wins.row[i].value[j]);
			printf("; ");
		}
	}

	// print all guaranteed loses
	printf("\nguaranteed lose : ");
	if (loses.size == 0)
		printf("-none-");
	else
	{	for (i=0; i<loses.size; ++i)
		{	printf("%c => \"%s\"", loses.row[i].key, loses.row[i].value[0]);
			for (j=1; j<loses.row[i].size; ++j)
				printf(", \"%s\"", loses.row[i].value[j]);
			printf("; ");
		}
	}

	// print all keep-alives
	printf("\nkeep-alive      : ");
	if (keepAlives.size == 0)
		printf("-none-");
	else
		for (i=0; i<keepAlives.size; ++i)
			printf("%c => \"%s\"; ", keepAlives.row[i].key, keepAlives.row[i].value[0]);

	printf("\n");
}

int main(int argc, char **argv)
{	if (argc < 2)
		printf("Please enter a substring as an argument!\n");
	else
		ghost(argv[1], dict);
	return 0;
}